package org.processmining.acceptingpetrinetreductionlayouter.models;

import java.util.ArrayList;
import java.util.Collection;

import org.processmining.acceptingpetrinetreductionlayouter.workspaces.ReductionWorkspace;
import org.processmining.framework.util.Pair;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class HorizontalBlock implements Block {

	private final int NOFLAYOUTS = 3;
	private Block[] layouts;
	private ReductionWorkspace ws;
	private double hSpace;

	public HorizontalBlock(Block leftLayout, Block midLayout, Block rightLayout, ReductionWorkspace ws) {
		hSpace = ws.parameters.getHorizontalSpace() * ws.SCALE;
		layouts = new Block[NOFLAYOUTS];
		layouts[0] = leftLayout;
		layouts[1] = midLayout;
		layouts[2] = rightLayout;
		this.ws = ws;
	}

	public int width(double hSpace) {
		this.hSpace = hSpace;
		int width = 0;
		for (int i = 0; i < NOFLAYOUTS; i++) {
			width += layouts[i].width(hSpace);
		}
		width += (NOFLAYOUTS - 1) * hSpace;
		return width;
	}

	public int[] height() {
		int[] height = new int[2];
		int[] heightLeft = layouts[0].height();
		int[] heightMid = layouts[1].height();
		int[] heightRight = layouts[2].height();
		height[0] = Math.max(heightLeft[0], Math.max(heightMid[0], heightRight[0]));
		height[1] = Math.max(heightLeft[1], Math.max(heightMid[1], heightRight[1]));
		return height;
	}

	public int getNofLayers() {
		int n = 0;
		for (int i = 0; i < NOFLAYOUTS; i++) {
			n += layouts[i].getNofLayers();
		}
		return n;
	}

	public void doLayout(ReductionWorkspace ws, Pair<Double, Double> position, boolean isReversed, double hSpace) {
		int[] height = height();
		if (isReversed) {
			for (int i = NOFLAYOUTS - 1; i >= 0; i--) {
				int[] subHeight = layouts[i].height();
				int offset = height[0] - subHeight[0];
				Pair<Double, Double> offsetPosition = new Pair<Double, Double>(position.getFirst(),
						position.getSecond() + offset);
				layouts[i].doLayout(ws, offsetPosition, isReversed, hSpace);
				position = new Pair<Double, Double>(position.getFirst() + layouts[i].width(hSpace) + hSpace,
						position.getSecond());
			}
		} else {
			for (int i = 0; i < NOFLAYOUTS; i++) {
				int[] subHeight = layouts[i].height();
				int offset = height[0] - subHeight[0];
				Pair<Double, Double> offsetPosition = new Pair<Double, Double>(position.getFirst(),
						position.getSecond() + offset);
				layouts[i].doLayout(ws, offsetPosition, isReversed, hSpace);
				position = new Pair<Double, Double>(position.getFirst() + layouts[i].width(hSpace) + hSpace,
						position.getSecond());
			}
		}
	}

	public boolean doSort(ReductionWorkspace ws) {
		return layouts[0].doSort(ws) || layouts[1].doSort(ws) || layouts[2].doSort(ws);
	}

	public Collection<PetrinetNode> getNodes(boolean addPrePostNodes) {
		Collection<PetrinetNode> nodes = new ArrayList<PetrinetNode>();
		for (int i = 0; i < NOFLAYOUTS; i++) {
			nodes.addAll(layouts[i].getNodes(addPrePostNodes));
		}
		return nodes;
	}

	public boolean doSortLoops(boolean isReversed) {
		return layouts[0].doSortLoops(isReversed) || layouts[1].doSortLoops(isReversed)
				|| layouts[2].doSortLoops(isReversed);
	}

	public Transition getFirstVisibleTransition(ReductionWorkspace ws) {
		for (int i = 0; i < NOFLAYOUTS; i++) {
			Transition transition = layouts[i].getFirstVisibleTransition(ws);
			if (transition != null) {
				return transition;
			}
		}
		return null;
	}

	public Transition getLastVisibleTransition(ReductionWorkspace ws) {
		for (int i = NOFLAYOUTS - 1; i >= 0; i--) {
			Transition transition = layouts[i].getLastVisibleTransition(ws);
			if (transition != null) {
				return transition;
			}
		}
		return null;
	}

	public void getTransitionPairs(ReductionWorkspace ws, Collection<Pair<Transition, Transition>> pairs) {
		for (int i = 0; i < NOFLAYOUTS; i++) {
			layouts[i].getTransitionPairs(ws, pairs);
		}
	}

	public void doPrint(int indent) {
		String s = "";
		for (int i = 0; i < indent; i++) {
			s += "\t";
		}
		int[] height = height();
		s += "Horizontal [" + width(hSpace) + "," + height[0] + "+" + height[1] + "]";
		System.out.println(s);
		layouts[0].doPrint(indent + 1);
		layouts[1].doPrint(indent + 1);
		layouts[2].doPrint(indent + 1);
	}
}
